@@ -109,7 +109,6 @@ class StandardCurve():
         else:
             model = self.model_dict[model_name]
 
-        print(model)
         smooth_x = linspace(
             self.concentration[0], self.concentration[-1], len(self.concentration)*2)
 
@@ -130,7 +129,6 @@ class StandardCurve():
 if __name__ == "__main__":
     import matplotlib.pyplot as plt
     from pyenzymekinetics.parameterestimator.helper.load_utitlity import calibration_conc, calibration_abso
-    print("bubu")
     obj = StandardCurve(concentration=calibration_conc, absorption=calibration_abso,
                         concentration_unit="mM", substance_name= "pNPGP")
     obj.visualize_fit()
