@@ -47,6 +47,7 @@ def to_concentration(
     allow_extrapolation:bool = False,
     ) -> ndarray:
 
+
     """Converts absobrance / peak area into concentration based on StandardCurve.
 
     Args:
@@ -68,9 +69,13 @@ def to_concentration(
         
 
 
-    shape = data.shape
-    data = data.flatten()
-    result = zeros(data.shape)
+    if isinstance(data, ndarray):
+        shape = data.shape
+        data = data.flatten()
+        result = zeros(data.shape)
+
+    else:
+        result = []
 
     if standard_curve_model_name is not None:
         model = standard_curve.model_dict[standard_curve_model_name]
@@ -82,8 +87,14 @@ def to_concentration(
 
     for value in range(len(data)):
         params["absorption"] = data[value]
-        result[value] = fsolve(equation, 0, params)
-    result = result.reshape(shape)
+
+        if isinstance(data, ndarray):
+            result[value] = fsolve(equation, 0, params)
+        else:
+            result.append(fsolve(equation, 0, params))
+
+    if isinstance(data, ndarray):
+        result = result.reshape(shape)
 
     # Check calibration bounds: 
     if max(data) > max(standard_curve.absorption):
