@@ -1,11 +1,12 @@
-from lib2to3.pygram import python_grammar_no_print_statement
 from pyenzymekinetics.utility.initial_parameters import get_initial_vmax, get_initial_Km
-from pyenzymekinetics.parameterestimator.models import KineticModel, kcatKm_inactivation, irreversible_model, irreversible_enzyme_inactication_model, competitive_inhibition_model, kcatKm, competitive_inhibition_enzyme_inactivation_model, uncompetitive_inhibition_model, uncompetitive_inhibition_enzyme_inactivation_model, partially_competitive_inhibition_model, noncompetitive_inhibition_model, substrate_inhibition_model, product_inhibition_model, substrate_inhibition_enzyme_inactivation_model
+from pyenzymekinetics.parameterestimator.models import KineticModel, kcatKm_inactivation, irreversible_model, irreversible_enzyme_inactication_model, competitive_inhibition_model, kcatKm, competitive_inhibition_enzyme_inactivation_model, uncompetitive_inhibition_model, uncompetitive_inhibition_enzyme_inactivation_model, partially_competitive_inhibition_model, noncompetitive_inhibition_model, substrate_inhibition_model, product_inhibition_model, substrate_inhibition_enzyme_inactivation_model, substrate_inactivation_model
 
 from pyenzyme import EnzymeMLDocument
+from pandas import DataFrame
+from IPython.display import display
 
 from matplotlib.cm import get_cmap
-from typing import Dict
+from typing import Dict, List
 from matplotlib import pyplot as plt
 from numpy import ndarray, array, zeros, max, tile
 import numpy as np
@@ -57,8 +58,6 @@ class EnzymeKinetics():
         self._w0 = self._get_w0()
         self.result_dict: dict = None
 
-        self.models: Dict[str, KineticModel] = self.initialize_models()
-
 
     def _check_is_substrate(self) -> bool:
         if self.substrate is not None:
@@ -92,7 +91,6 @@ class EnzymeKinetics():
             )
 
     def _check_inhibitor(self):
-        print(self.inhibitor)
         """Make inhibitor array same length as init substrate array"""
         #if self.inhibitor == None:
          #   self.inhibitor = np.array([0])
@@ -103,7 +101,7 @@ class EnzymeKinetics():
             pass
         else:
             raise Exception(
-                "Inconsistent array Lengths. Enzyme array length should be one or equivalent to init_Substrate array length."
+                f"Inconsistent array lengths. Inhibitor array length ({len(self.inhibitor)}) should be one or equivalent to init_Substrate array length ({len(self.init_substrate)})."
             )
 
     def calculate_substrate(self) -> ndarray:
@@ -159,7 +157,32 @@ class EnzymeKinetics():
             return get_initial_vmax(self.substrate, self.time) / self.enzyme
 
 
-    def initialize_models(self) -> Dict[str, KineticModel]:
+    def fit_models(
+        self, 
+        models: str = None,
+        initial_substrates: list = None, 
+        time_steps: int = None,
+        ) -> Dict[str, KineticModel]:
+
+        pos = np.array([])
+        if initial_substrates != None:
+
+            for concentration in initial_substrates:
+                if concentration not in self.init_substrate:
+                    raise ValueError(
+                        f"{concentration} not found in initial substrate concentrations. \nInitial substrate concentrations are {np.unique(self.init_substrate)}")
+                else:
+                    pos = np.append(pos, np.where(self.init_substrate == concentration)[0])
+            pos = pos.astype(int)
+        
+            self.substrate = self.substrate[pos,:time_steps]
+            self.product = self.product[pos,:time_steps]
+            self.enzyme = self.enzyme[pos]
+            self.init_substrate = self.init_substrate[pos]
+            self.time = self.time[:time_steps]
+            self.inhibitor = self.inhibitor[pos]
+
+
         irreversible_MM = KineticModel(
             name="irreversible Michaelis Menten",
             params=(""),
@@ -217,8 +240,17 @@ class EnzymeKinetics():
             Km_initial=get_initial_Km(self.substrate, self.time),
             model=substrate_inhibition_enzyme_inactivation_model
         )
-        
 
+        substrate_inactivation = KineticModel(
+            name="Substrate dependent enzyme inactivation",
+            params=["K_ie"],
+            w0={"cS": self.init_substrate, "cE": self.enzyme,
+                "cP": self.product, "cI": self.inhibitor},
+            kcat_initial=self._get_kcat(),
+            Km_initial=get_initial_Km(self.substrate, self.time),
+            model=substrate_inactivation_model
+        )
+        
         product_inhibition = KineticModel(
             name="product inhibition",
             params=["K_ic"],
@@ -289,34 +321,37 @@ class EnzymeKinetics():
             model=noncompetitive_inhibition_model
         )
 
-        """
-        irrev_MM_subabs = KineticModel(
-            name="irreversible Michaelis Menten with absorbing substrate",
-            params=("a"),
-            w0={"cS": self.init_substrate, "cE": self.enzyme, "cP": self.product, "cS0": self.init_substrate},
-            kcat_initial=self._get_kcat(),
-            Km_initial=get_initial_Km(self.substrate, self.time),
-            model=subabs_menten_irreversible
-        )
-        """
-        kinetic_model_dict: Dict[str, KineticModel] = {
-            irreversible_MM.name: irreversible_MM,
-            irrev_MM_enz_inact.name: irrev_MM_enz_inact,
-            kcat_km.name: kcat_km,
-            kcat_km_enz_inactivation.name: kcat_km_enz_inactivation,
-            substrate_inhibition.name: substrate_inhibition,
-            product_inhibition.name: product_inhibition,
-            competitive_inhibition.name: competitive_inhibition,
-            competitive_inhibition_enzyme_inactivation.name: competitive_inhibition_enzyme_inactivation,
-            uncompetitive_inhibition.name: uncompetitive_inhibition,
-            uncompetitive_inhibition_enzyme_inactivation.name: uncompetitive_inhibition_enzyme_inactivation,
-            uncompetitive_inhibition_enzyme_inactivation.name: uncompetitive_inhibition_enzyme_inactivation,
-            noncompetitive_inhibition.name: noncompetitive_inhibition,
-            substrate_inhibition_enz_inactivation.name: substrate_inhibition_enz_inactivation
-            #irrev_MM_subabs.name: irrev_MM_subabs
-        }
+        if np.all(self.inhibitor == 0):
+            kinetic_model_dict: Dict[str, KineticModel] = {
+                irreversible_MM.name: irreversible_MM,
+                irrev_MM_enz_inact.name: irrev_MM_enz_inact,
+                #kcat_km.name: kcat_km,
+                #kcat_km_enz_inactivation.name: kcat_km_enz_inactivation,
+                substrate_inhibition.name: substrate_inhibition,
+                substrate_inhibition_enz_inactivation.name: substrate_inhibition_enz_inactivation,
+                product_inhibition.name: product_inhibition,
+            }
+        else:
+            kinetic_model_dict: Dict[str, KineticModel] = {
+                irreversible_MM.name: irreversible_MM,
+                irrev_MM_enz_inact.name: irrev_MM_enz_inact,
+                #kcat_km.name: kcat_km,
+                #kcat_km_enz_inactivation.name: kcat_km_enz_inactivation,
+                substrate_inhibition.name: substrate_inhibition,
+                substrate_inhibition_enz_inactivation.name: substrate_inhibition_enz_inactivation,
+                substrate_inactivation.name: substrate_inactivation,
+                product_inhibition.name: product_inhibition,
+                competitive_inhibition.name: competitive_inhibition,
+                competitive_inhibition_enzyme_inactivation.name: competitive_inhibition_enzyme_inactivation,
+                uncompetitive_inhibition.name: uncompetitive_inhibition,
+                uncompetitive_inhibition_enzyme_inactivation.name: uncompetitive_inhibition_enzyme_inactivation,
+                noncompetitive_inhibition.name: noncompetitive_inhibition,
+                partially_competitive_inhibition.name: partially_competitive_inhibition
+            }
 
-        return kinetic_model_dict
+        self.models: Dict[str, KineticModel] = kinetic_model_dict
+
+        self.run_models()
 
     def evaluate_aic(self):
         names = []
@@ -331,7 +366,9 @@ class EnzymeKinetics():
         return result_dict
 
 
-    def fit_models(self):
+    def run_models(self):
+
+
         for kineticmodel in self.models.values():
 
             def g(t, w0, params):
@@ -369,11 +406,12 @@ class EnzymeKinetics():
 
                 return resid.flatten()
 
-            print(kineticmodel.name)
             kineticmodel.result = minimize(residual, kineticmodel.parameters, args=(
                 self.time, self.substrate), method='leastsq', nan_policy='omit')
 
-        self.result_dict = self.evaluate_aic()
+        self.result_dict = self._result_overview()
+
+        display(self.result_dict)
 
 
     def visualize_fit(self, model_name: str = None, path: str = None, title:str = None, visualize_species: str = None, **plt_kwargs):
@@ -383,7 +421,6 @@ class EnzymeKinetics():
             "S": [self.substrate,0, self.substrate_name], # Substrate
             "P": [self.product, 2, self.product_name], # Product
         }
-        print("mashallah")
 
         if visualize_species is None:
             if self._is_substrate:
@@ -395,12 +432,11 @@ class EnzymeKinetics():
 
 
         # TODO: add file directory for save
-        best_model = next(iter(self.result_dict))
+        best_model = self.result_dict.index[0]
         if model_name is None:
             model_name = best_model
 
         model = self.models[model_name]
-        report_fit(model.result)
 
         def g(t, w0, params):
 
@@ -457,11 +493,14 @@ class EnzymeKinetics():
                     new_labels.append(label)
                     new_handles.append(handle)
 
-        plt.legend(title = "initial substrate [mM]", handles=new_handles, labels=new_labels)
+        plt.legend(title = f"initial substrate [{self.unit}]", handles=new_handles, labels=new_labels)
         if path != None:
             plt.savefig(path, format="svg")
         plt.show()
 
+        report_fit(model.result)
+
+
 
     @classmethod
     def from_EnzymeML(
@@ -484,7 +523,8 @@ class EnzymeKinetics():
             EnymeKinetics: Initialized EnzymeKinetics object
         """
 
-
+        # TODO iterate over replicates and get information
+        # dont assume same number of replicates for ech measurement
         substrate_data = np.array([])
         product_data = np.array([])
         protein_data = np.array([])
@@ -532,12 +572,14 @@ class EnzymeKinetics():
         if isinstance(substrate_data, np.ndarray):
             substrate_data = substrate_data.reshape((n_measurements*n_replicates,len(time)))
         if isinstance(product_data, np.ndarray):
-            product_data = substrate_data.reshape((n_measurements*n_replicates,len(time)))
+            product_data = product_data.reshape((n_measurements*n_replicates,len(time)))
 
         substrate_name = enzymemldoc.getReactant(substrate_id).name
         product_name = enzymemldoc.getReactant(product_id).name
         time_unit = measurement.getReactant(measured_species).replicates[0].time_unit
         unit = measurement.getReactant(measured_species).replicates[0].data_unit
+        if inhibitor_id != None:
+            inhibitor_data = np.repeat(inhibitor_data, n_replicates)
 
         instance = cls(
             time=time,
@@ -553,6 +595,41 @@ class EnzymeKinetics():
         )
         return instance
 
+    def _result_overview(self):
+
+        parameter_mapper = {
+            "k_cat": f"Kcat [1/{self.time_unit}]",
+            "Km": f"Km [{self.unit}]",
+            "K_ic": "Ki competitive",
+            "K_iu": "Ki uncompetitive",
+            "K_ie": f"Ki time-dep enz-inactiv. [1/{self.time_unit}]",
+        }
+
+        result_dict = {}
+        for model in self.models.values():
+            name = model.name
+            aic = round(model.result.aic)
+
+            parameter_dict = {}
+            for parameter in model.result.params.values():
+                name = parameter_mapper[parameter.name]
+                value = parameter.value
+                stderr = parameter.stderr
+
+                percentual_stderr = stderr / value * 100
+
+                if name.startswith("Ki time-dep"):
+                    parameter_dict[name] = f"{value:.5f} +/- {percentual_stderr:.2f}%"
+                else:
+                    parameter_dict[name] = f"{value:.2f} +/- {percentual_stderr:.2f}%"
+
+
+            result_dict[model.name] = {"AIC": aic, **parameter_dict}
+
+        df = DataFrame.from_dict(result_dict).T.sort_values("AIC", ascending=True)
+        df.fillna('-', inplace=True)
+
+        return df
 
 
 if __name__ == "__main__":
@@ -561,11 +638,17 @@ if __name__ == "__main__":
     #from pyenzymekinetics.parameterestimator.helper.load_utitlity import *
     from pyenzymekinetics.calibrator.standardcurve import StandardCurve
     from pyenzymekinetics.calibrator.utility import to_concentration
+    import pandas as pd
     import numpy as np
 
     import pyenzyme as pe
 
-    enzmldoc = pe.EnzymeMLDocument.fromFile("/Users/maxhaussler/Dropbox/master_thesis/data/marwa/round4/DocTest.omex")
+    #enzmldoc = pe.EnzymeMLDocument.fromFile("/Users/maxhaussler/Dropbox/master_thesis/data/marwa/round4/DocTest.omex")
+    enzmldoc = pe.EnzymeMLDocument.fromFile("/Users/maxhaussler/Dropbox/master_thesis/data/sandile/pySCeS/Ngubane_ABTS.omex")
+
     mm = EnzymeKinetics.from_EnzymeML(enzmldoc)
-    mm.fit_models()
-    mm.visualize_fit(visualize_species="P", s=12)
\ No newline at end of file
+    mm.fit_models(initial_substrates=[5,50,110], time_steps=4)
+    #print(mm._result_overview())
+    pd.DataFrame.from_dict(mm._result_overview())
+    #mm.visualize_fit()
+    
\ No newline at end of file
