@@ -100,6 +100,20 @@ def substrate_inhibition_model(w0: tuple, t, params) -> tuple:
 
     return (dc_S, dc_E, dc_P, dc_I)
 
+def substrate_inactivation_model(w0: tuple, t, params) -> tuple:
+    cS, cE, cP, cI = w0
+
+    k_cat = params['k_cat'].value
+    Km = params['Km'].value
+    K_ie = params["K_ie"].value
+
+    dc_S = -k_cat * cE * (cS) / (Km+cS)
+    dc_E = cE * cS * K_ie
+    dc_P = -dc_S
+    dc_I = 0
+
+    return (dc_S, dc_E, dc_P, dc_I)
+
 def substrate_inhibition_enzyme_inactivation_model(w0: tuple, t, params) -> tuple:
     cS, cE, cP, cI = w0
 
